/*-
 * Copyright (C)2012 @BABOLO http://www.babolo.ru/
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C)2012 @BABOLO http://www.babolo.ru/\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: switch.c,v 1.2 2012/08/21 13:42:37 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <sysexits.h>
#include <strings.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <babolo/BLINflag.h>
#include "transposit.h"

#define TOBY(A) (((A) + 7) >> 3)
int
/*****************************************************************************************************
 *    ,    0.                            *
 *****************************************************************************************************/
transposit_swcmp(transposit_switch *swr0, transposit_switch *swr1) {
    int ex = 0;
    size_t i;

    if  (swr0->nou < swr1->nou) {
        ex = -1;
        goto out;
    } else if (swr0->nou > swr1->nou) {
        ex = 1;
        goto out;
    }
    for (i = 0; i < swr0->nou; ++i) {
        if  (swr0->swt[i] < swr1->swt[i]) {
            ex = -1;
            goto out;
        } else if (swr0->swt[i] > swr1->swt[i]) {
            ex = 1;
            goto out;
    }   }
out:
    return(ex);
}

int
/*****************************************************************************************************
 *     swr   sdp,                                     *
 *    codes  coden .                                            *
 *      ,      .             *
 *****************************************************************************************************/
transposit_swdepo(transposit_deposit **sdp, transposit_switch *swr, void *codes, size_t coden) {
    ifBLIN_FLAG = swr->flags;
    transposit_depocodes *dc;
    transposit_deposit *dp;
    int ex = 0;

    for (dp = *sdp;; dp = dp->next) {
        if  (!dp) {
            if  (!(dp = calloc(1, sizeof(transposit_deposit)))) {
                errno = ENOMEM;
                ifBLIN_QW0("#3");
                ex = -EX_OSERR;
                goto out;
            }
            dp->next = *sdp;
            *sdp = dp;
            dp->swc = swr;
            if  (!(dp->codes = calloc(1, TOBY(coden) + sizeof(transposit_depocodes)))) {
                errno = ENOMEM;
                ifBLIN_QW0("#4");
                ex = -EX_OSERR;
                goto out;
            }
            dp->codes->coden = coden;
            bcopy(codes, dp->codes->codes, TOBY(coden));
            ex = 1;
            goto out;
        }
        if  (!transposit_swcmp(swr, dp->swc)) {
            for (dc = dp->codes;; dc = dc->next) {
                if  (!dc) {
                    if  (!(dc = calloc(1, sizeof(transposit_depocodes)))) {
                        errno = ENOMEM;
                        ifBLIN_QW0("#5");
                        ex = -EX_OSERR;
                        goto out;
                    }
                    dc->next = dp->codes;
                    dp->codes = dc;
                    dc->coden = coden;
                    bcopy(codes, dc->codes, TOBY(coden));
                    goto out;
    }   }   }   }
out:
    return(ex);
}

void
/*****************************************************************************************************
 *        swr   o.                             *
 *      .                                                             *
 *         , *  ,                    *
 *     .                                                                *
 *****************************************************************************************************/
transposit_swprint(transposit_switch *swr, FILE *o) {
    size_t i, l;

    fprintf(o, "      ");
    for (l = 0; l < swr->nin; ++l) fprintf(o, "%2"BLIN_U, BLIN_I(l));
    fprintf(o, "\n");
    for (i = 0; i < swr->nou; ++i) {
        fprintf(o, "   %2"BLIN_U":", BLIN_I(i));
        for (l = 0; l < swr->nin; ++l) {
            fprintf(o, " %c", (l == swr->swt[i]) ? '*' : '.');
        }
        fprintf(o, "\n");
}   }

void
/*****************************************************************************************************
 *        o,    :       *
 *  -          ;                                 *
 *  -      transposit_swprint()                                *
 *****************************************************************************************************/
transposit_deprint(transposit_deposit *sdp, FILE *o) {
    transposit_depocodes *dc;
    size_t i, l;

    for (i = 0; sdp; ++i, sdp = sdp->next) {
        fprintf(o, "%5"BLIN_U":", BLIN_I(i));
        for (dc = sdp->codes; dc; dc = dc->next) {
            fprintf(o, " ");
            for (l = 0; l < TOBY(dc->coden); ++l) fprintf(o, "%02X", dc->codes[l]);
        }
        fprintf(o, "\n");
        transposit_swprint(sdp->swc, o);
}   }
